#ifndef TOOLS_H
#define TOOLS_H

#endif // TOOLS_H

#include "Eigen/Dense"

using namespace Eigen;

inline void makeCrossProductMatrix(Vector3f const& v, Matrix3f& m)
{
    assert(v.size() == 3);
    m(0,0) = 0;     m(0,1) = -v[2]; m(0,2) = v[1];
    m(1,0) = v[2];  m(1,1) = 0;     m(1,2) = -v[0];
    m(2,0) = -v[1]; m(2,1) = v[0];  m(2,2) = 0;
}

inline void makeRodriguesRotationMatrix(Vector3f const& omega, Matrix3f& R)
{
    assert(omega.size() == 3);

    double const theta = omega.norm();
    R.setIdentity();
    if (fabs(theta) > 1e-6)
    {
       Matrix3f J, J2;
       makeCrossProductMatrix(omega, J);
       J2 = J * J;
       double const c1 = sin(theta)/theta;
       double const c2 = (1-cos(theta))/(theta*theta);
       for (int i = 0; i < 3; ++i)
          for (int j = 0; j < 3; ++j)
              R(i,j) += c1*J(i,j) + c2*J2(i,j);
    }

}

inline float
getRotationMagnitude(Matrix3f const& R)
{
    R.trace();
    double cos_theta = 0.5 * (R(0,0) + R(1,1) + R(2,2) - 1.0);
   cos_theta = std::max(-1.0, std::min(1.0, cos_theta));
   return acos(cos_theta);
}

template <typename T> inline T sqr(T x) { return x*x; }

inline Vector3f
getRotationAxis(Matrix3f const& R)
{
   Vector3f axis(0.0, 0.0, 1.0);

   double const two_sin_theta = sqrt(sqr(R(1,0) - R(0,1)) + sqr(R(0,2) - R(2,0)) + sqr(R(2,1) - R(1,2)));

   if (fabs(two_sin_theta) > 1e-10)
   {
      axis(0) = (R(2,1) - R(1,2)) / two_sin_theta;
      axis(1) = (R(0,2) - R(2,0)) / two_sin_theta;
      axis(2) = (R(1,0) - R(0,1)) / two_sin_theta;
   }
   return axis;
}
